package com.jiuqi.util.crypto;

import java.io.OutputStream;
import java.io.FileOutputStream;
import com.jiuqi.util.JqLib;
import java.io.FileInputStream;
import java.io.File;
import java.io.ByteArrayOutputStream;
import javax.crypto.Cipher;
import java.security.Key;
import java.security.spec.RSAPrivateKeySpec;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.KeyFactory;
import java.security.interfaces.RSAPublicKey;
import java.security.SecureRandom;
import java.security.Provider;
import java.security.KeyPairGenerator;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import java.security.KeyPair;

public class RSAUtil
{
    public static KeyPair generateKeyPair() throws EncryptException {
        try {
            final KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA", (Provider)new BouncyCastleProvider());
            keyPairGen.initialize(1024, new SecureRandom());
            final KeyPair keyPair = keyPairGen.genKeyPair();
            return keyPair;
        }
        catch (final Exception e) {
            throw new EncryptException(e.getMessage());
        }
    }
    
    public static RSAPublicKey generateRSAPublicKey(final byte[] modulus, final byte[] publicExponent) throws EncryptException {
        KeyFactory keyFac = null;
        try {
            keyFac = KeyFactory.getInstance("RSA", (Provider)new BouncyCastleProvider());
        }
        catch (final NoSuchAlgorithmException ex) {
            throw new EncryptException(ex.getMessage());
        }
        final RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(modulus), new BigInteger(publicExponent));
        try {
            return (RSAPublicKey)keyFac.generatePublic(pubKeySpec);
        }
        catch (final InvalidKeySpecException ex2) {
            throw new EncryptException(ex2.getMessage());
        }
    }
    
    public static RSAPrivateKey generateRSAPrivateKey(final byte[] modulus, final byte[] privateExponent) throws EncryptException {
        KeyFactory keyFac = null;
        try {
            keyFac = KeyFactory.getInstance("RSA", (Provider)new BouncyCastleProvider());
        }
        catch (final NoSuchAlgorithmException ex) {
            throw new EncryptException(ex.getMessage());
        }
        final RSAPrivateKeySpec priKeySpec = new RSAPrivateKeySpec(new BigInteger(modulus), new BigInteger(privateExponent));
        try {
            return (RSAPrivateKey)keyFac.generatePrivate(priKeySpec);
        }
        catch (final InvalidKeySpecException ex2) {
            throw new EncryptException(ex2.getMessage());
        }
    }
    
    public static byte[] encrypt(final Key key, final byte[] data) throws EncryptException {
        try {
            final Cipher cipher = Cipher.getInstance("RSA", (Provider)new BouncyCastleProvider());
            cipher.init(1, key);
            final int blockSize = cipher.getBlockSize();
            final int outputSize = cipher.getOutputSize(data.length);
            final int leavedSize = data.length % blockSize;
            final int blocksSize = (leavedSize != 0) ? (data.length / blockSize + 1) : (data.length / blockSize);
            final byte[] raw = new byte[outputSize * blocksSize];
            for (int i = 0; data.length - i * blockSize > 0; ++i) {
                if (data.length - i * blockSize > blockSize) {
                    cipher.doFinal(data, i * blockSize, blockSize, raw, i * outputSize);
                }
                else {
                    cipher.doFinal(data, i * blockSize, data.length - i * blockSize, raw, i * outputSize);
                }
            }
            return raw;
        }
        catch (final Exception e) {
            throw new EncryptException(e.getMessage());
        }
    }
    
    public static byte[] decrypt(final Key key, final byte[] raw) throws EncryptException {
        try {
            final Cipher cipher = Cipher.getInstance("RSA", (Provider)new BouncyCastleProvider());
            cipher.init(2, key);
            final int blockSize = cipher.getBlockSize();
            final ByteArrayOutputStream bout = new ByteArrayOutputStream(64);
            for (int j = 0; raw.length - j * blockSize > 0; ++j) {
                bout.write(cipher.doFinal(raw, j * blockSize, blockSize));
            }
            return bout.toByteArray();
        }
        catch (final Exception e) {
            throw new EncryptException(e.getMessage());
        }
    }
    
    public static void main(final String[] args) throws Exception {
        File file = new File("d:\\test.html");
        final FileInputStream in = new FileInputStream(file);
        final ByteArrayOutputStream bout = new ByteArrayOutputStream();
        byte[] tmpbuf = new byte[1024];
        int count = 0;
        while ((count = in.read(tmpbuf)) != -1) {
            bout.write(tmpbuf, 0, count);
            tmpbuf = new byte[1024];
        }
        in.close();
        final byte[] orgData = bout.toByteArray();
        final KeyPair keyPair = generateKeyPair();
        final RSAPublicKey pubKey = (RSAPublicKey)keyPair.getPublic();
        final RSAPrivateKey priKey = (RSAPrivateKey)keyPair.getPrivate();
        final byte[] pubModBytes = pubKey.getModulus().toByteArray();
        final byte[] pubPubExpBytes = pubKey.getPublicExponent().toByteArray();
        final byte[] priModBytes = priKey.getModulus().toByteArray();
        final byte[] priPriExpBytes = priKey.getPrivateExponent().toByteArray();
        final RSAPublicKey recoveryPubKey = generateRSAPublicKey(pubModBytes, pubPubExpBytes);
        final RSAPrivateKey recoveryPriKey = generateRSAPrivateKey(priModBytes, priPriExpBytes);
        System.out.println(JqLib.bytesToHexString(priModBytes).toString());
        System.out.println(JqLib.bytesToHexString(priPriExpBytes).toString());
        System.out.println(JqLib.bytesToHexString(pubModBytes).toString());
        System.out.println(JqLib.bytesToHexString(pubPubExpBytes).toString());
        final byte[] raw = encrypt(recoveryPriKey, orgData);
        file = new File("d:\\encrypt_result.dat");
        OutputStream out = new FileOutputStream(file);
        out.write(raw);
        out.close();
        final byte[] data = decrypt(recoveryPubKey, raw);
        file = new File("d:\\decrypt_result.html");
        out = new FileOutputStream(file);
        out.write(data);
        out.flush();
        out.close();
    }
}
